@@ -25,6 +25,7 @@
 #include <libsecret/secret.h>
 #include <json-glib/json-glib.h>
 
+#include "goaauthflowbutton.h"
 #include "goaprovider.h"
 #include "goaproviderdialog.h"
 #include "goautils.h"
@@ -826,23 +827,13 @@ parse_request_uri (GoaOAuth2Provider  *self,
   AccountData *data = g_task_get_task_data (task);
   g_autoptr(GHashTable) key_value_pairs = NULL;
   g_autoptr(GUri) uri = NULL;
-  g_autoptr(GUri) redirect_uri = NULL;
   const char *fragment;
   const char *oauth2_error;
   const char *query;
 
   g_assert (error == NULL || *error == NULL);
 
-  uri = g_uri_parse (requested_uri, G_URI_FLAGS_ENCODED | G_URI_FLAGS_PARSE_RELAXED, error);
-  if (uri == NULL)
-    return FALSE;
-
-  redirect_uri = g_uri_parse (data->redirect_uri, G_URI_FLAGS_ENCODED | G_URI_FLAGS_PARSE_RELAXED, error);
-  if (redirect_uri == NULL)
-    return FALSE;
-
-  if (g_strcmp0 (g_uri_get_scheme (uri), g_uri_get_scheme (redirect_uri)) != 0
-      || g_strcmp0 (g_uri_get_path (uri), g_uri_get_path (redirect_uri)) != 0)
+  if (!g_str_has_prefix (requested_uri, data->redirect_uri))
     {
       g_set_error (error,
                    GOA_ERROR,
@@ -852,6 +843,10 @@ parse_request_uri (GoaOAuth2Provider  *self,
       return FALSE;
     }
 
+  uri = g_uri_parse (requested_uri, G_URI_FLAGS_ENCODED | G_URI_FLAGS_PARSE_RELAXED, error);
+  if (uri == NULL)
+    return FALSE;
+
   /* Three cases:
    * 1) we can either have the backend handle the URI for us, or
    * 2) we can either have the access_token and other information
@@ -999,10 +994,20 @@ identity_from_auth (GoaOAuth2Provider  *self,
 /* ---------------------------------------------------------------------------------------------------- */
 
 static void
-on_copylink_activated (AccountData *data)
+on_copy_activated (GoaAuthflowButton *widget,
+                   AccountData       *data)
 {
   data->flags |= GOA_AUTH_FLOW_DO_NOT_LAUNCH_URI;
   goa_provider_dialog_set_state (data->dialog, GOA_DIALOG_BUSY);
+  goa_authflow_button_set_active (widget, TRUE);
+}
+
+static void
+on_link_activated (GoaAuthflowButton *widget,
+                   AccountData       *data)
+{
+  goa_provider_dialog_set_state (data->dialog, GOA_DIALOG_BUSY);
+  goa_authflow_button_set_active (widget, TRUE);
 }
 
 static void
@@ -1011,55 +1016,28 @@ create_account_details_ui (GoaProvider *provider,
                            gboolean     new_account)
 {
   GoaProviderDialog *dialog = GOA_PROVIDER_DIALOG (data->dialog);
-  GtkWidget *content, *content_box;
-  GtkWidget *buttons;
+  GtkWidget *content;
   GtkWidget *button;
-  GtkWidget *copy_button, *copy_desc;
   g_autofree char *provider_name = NULL;
   g_autofree char *description = NULL;
 
   provider_name = goa_provider_get_provider_name (provider, NULL);
   description = g_strdup_printf (_("Sign in to %s with your browser"), provider_name);
-  content_box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 18);
 
-  buttons = gtk_list_box_new ();
-  gtk_widget_add_css_class (buttons, "boxed-list-separate");
-  gtk_list_box_set_selection_mode (GTK_LIST_BOX (buttons), GTK_SELECTION_NONE);
-  gtk_box_append (GTK_BOX (content_box), buttons);
-
-  button = g_object_new (ADW_TYPE_BUTTON_ROW,
-                         "title", _("_Sign In…"),
-                         "use-underline", TRUE,
-                         NULL);
-  gtk_widget_add_css_class (button, "suggested-action");
-  gtk_list_box_append (GTK_LIST_BOX (buttons), button);
-
-  /* When "Copy Link" is clicked, we pin the flags for the auth flow */
-  copy_button = g_object_new (ADW_TYPE_BUTTON_ROW,
-                              "title", _("_Copy Link"),
-                              "use-underline", TRUE,
-                              "start-icon-name", "edit-copy-symbolic",
-                              NULL);
-  g_signal_connect_swapped (copy_button,
-                            "activated",
-                            G_CALLBACK (on_copylink_activated),
-                            data);
-  gtk_list_box_append (GTK_LIST_BOX (buttons), copy_button);
-
-  copy_desc = gtk_label_new (_("Copy the authorization URL to continue with a specific web browser."));
-  gtk_label_set_justify (GTK_LABEL (copy_desc), GTK_JUSTIFY_CENTER);
-  gtk_label_set_wrap (GTK_LABEL (copy_desc), TRUE);
-  gtk_widget_set_halign (copy_desc, GTK_ALIGN_CENTER);
-  gtk_widget_add_css_class (copy_desc, "caption");
-  gtk_accessible_update_relation (GTK_ACCESSIBLE (copy_button),
-                                  GTK_ACCESSIBLE_RELATION_DESCRIBED_BY, copy_desc, NULL,
-                                  -1);
-  gtk_box_append (GTK_BOX (content_box), copy_desc);
+  button = goa_authflow_button_new ();
+  g_signal_connect (button,
+                    "copy-activated",
+                    G_CALLBACK (on_copy_activated),
+                    data);
+  g_signal_connect (button,
+                    "link-activated",
+                    G_CALLBACK (on_link_activated),
+                    data);
 
   content = g_object_new (ADW_TYPE_STATUS_PAGE,
                           "icon-name",   "web-browser-symbolic",
                           "description", description,
-                          "child",       content_box,
+                          "child",       button,
                           NULL);
   goa_provider_dialog_push_content (dialog, NULL, content);
 
@@ -1172,7 +1150,6 @@ oauth2_handler_dbus_method_call (GDBusConnection       *connection,
           g_debug ("Received OAuth2 response for client ID \"%s\"", client_id);
 
           g_dbus_method_invocation_return_value (invocation, NULL);
-          authorize_uri_task_complete (NULL, task);
           g_task_return_pointer (task, g_strdup (response), g_free);
         }
       else
@@ -1184,10 +1161,10 @@ oauth2_handler_dbus_method_call (GDBusConnection       *connection,
                                                  G_DBUS_ERROR_INVALID_ARGS,
                                                  "Invalid URI \"%s\"",
                                                  response);
-          authorize_uri_task_complete (NULL, task);
           g_task_return_error (task, g_steal_pointer (&error));
         }
 
+      authorize_uri_task_complete (NULL, task);
       return;
     }
 
@@ -1216,8 +1193,8 @@ authorize_uri_launch_uri_cb (GObject      *object,
 
   if (!g_app_info_launch_default_for_uri_finish (result, &error))
     {
-      authorize_uri_task_complete (NULL, task);
       g_task_return_error (task, g_steal_pointer (&error));
+      authorize_uri_task_complete (NULL, task);
     }
 }
 
@@ -1249,8 +1226,8 @@ on_oauth2_bus_acquired (GDBusConnection *connection,
 
   if (data->register_object_id == 0)
     {
-      authorize_uri_task_complete (NULL, task);
       g_task_return_error (task, g_steal_pointer (&error));
+      authorize_uri_task_complete (NULL, task);
     }
 }
 
@@ -1285,11 +1262,11 @@ on_oauth2_name_lost (GDBusConnection *connection,
   else
     g_warning ("%s(): Failed to own %s on the session bus", G_STRFUNC, name);
 
-  authorize_uri_task_complete (NULL, task);
   g_task_return_new_error_literal (task,
                                    GOA_ERROR,
                                    GOA_ERROR_FAILED,
                                    _("Service not available"));
+  authorize_uri_task_complete (NULL, task);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
