@@ -503,7 +503,7 @@ goa_provider_dialog_set_state (GoaProviderDialog *self,
         adw_banner_set_revealed (ADW_BANNER (banner), state == GOA_DIALOG_ERROR);
 
       content = g_object_get_data (G_OBJECT (page), "goa-dialog-content");
-      if (content != NULL)
+      if (ADW_IS_PREFERENCES_PAGE (content))
         gtk_widget_set_sensitive (content, state != GOA_DIALOG_BUSY);
     }
 
@@ -888,13 +888,6 @@ goa_provider_dialog_push_account (GoaProviderDialog *self,
                     self);
   adw_preferences_group_add (ADW_PREFERENCES_GROUP (group), row);
 
-  // Gvfs requires a user and host for OneDrive mount spec, so disallow
-  // custom presentation identities for this provider
-  if (g_str_equal (goa_provider_get_provider_type (self->provider), GOA_MS_GRAPH_NAME))
-    {
-      gtk_editable_set_editable (GTK_EDITABLE (row), FALSE);
-    }
-
   revealer = gtk_revealer_new ();
   gtk_revealer_set_transition_type (GTK_REVEALER (revealer), GTK_REVEALER_TRANSITION_TYPE_CROSSFADE);
   adw_preferences_group_add (ADW_PREFERENCES_GROUP (group), revealer);
